from collections import namedtuple
import numpy as np

Mesh = namedtuple('Mesh', ['domain', 'sommets', 'centres', 'mes', 'dist'])

def make_uniform_mesh(L, N):
    h = L / N
    domain = (0, L)
    sommets = np.linspace(0, L, N+1)
    centres = np.linspace(h, L, N) - h/2
    mes = h * np.ones(N)
    dist = h * np.ones(N+1)
    dist[0] = h/2
    dist[N] = h/2

    return Mesh(domain, sommets, centres, mes, dist)


def make_random_mesh(L, N, eta=0.1):
    domain = (0, L)
    x=np.linspace(0+(1/(N-1)),L*(1-1/(N-1)),N-1)+eta*(2*np.random.rand(N-1)-1)*L/(N-1)
    x = np.sort(x) # useful ?
    sommets = np.hstack(([0], x, [L]))
    # On s'arrange pour que L/2 soit un sommet du maillage
    indice = np.min(np.argmin(np.abs(sommets - L/2)))
    sommets[indice]=L/2;

    centres = (sommets[1:] + sommets[:-1])/2
    mes = sommets[1:] - sommets[:-1]
    dist = centres[1:] - centres[:-1]
    dist = np.hstack(([centres[0]], dist, [L-centres[N-1]]))
    
    return Mesh(domain, sommets, centres, mes, dist)

def normeL2(m, u):
    return np.sqrt(np.sum(m.mes*u**2))

